@@ -1,12 +1,13 @@
 import difflib
 import itertools
 import math
-import os.path
+import os
 import re
 import subprocess
 from collections import namedtuple
 from contextlib import contextmanager
 from datetime import datetime
+from datetime import timezone
 from grp import getgrnam
 from subprocess import call
 
@@ -16,118 +17,174 @@ from Crypto.PublicKey import RSA
 import ocflib.account.search as search
 import ocflib.account.utils as utils
 import ocflib.account.validators as validators
-import ocflib.constants as constants
 from ocflib.infra.kerberos import create_kerberos_principal_with_keytab
-from ocflib.infra.ldap import create_ldap_entry_with_keytab
+from ocflib.infra.kerberos import get_kerberos_principal_with_keytab
+from ocflib.infra.ldap import create_ldap_entry
 from ocflib.infra.ldap import ldap_ocf
+from ocflib.infra.ldap import OCF_LDAP_PEOPLE
+from ocflib.misc.mail import jinja_mail_env
 from ocflib.misc.mail import send_mail
 from ocflib.misc.validators import valid_email
+from ocflib.printing.quota import SEMESTERLY_QUOTA
 
 
-def create_account(request, creds, report_status):
-    """Create an account as idempotently as possible."""  # TODO: docstring
+_KNOWN_UID = 71136
+BAD_WORDS = frozenset((
+    'anal', 'anus', 'arse', 'ass', 'bastard', 'bitch', 'biatch', 'bloody', 'blowjob', 'bollock',
+    'bollok', 'boner', 'chink', 'clit', 'cock', 'coon', 'cunt', 'damn', 'dick', 'dildo', 'douche',
+    'dyke', 'fag', 'fellate', 'fellatio', 'felching', 'fuck', 'flange', 'hell', 'homo', 'jerk', 'jizz', 'kike',
+    'labia', 'muff', 'nigger', 'nigga', 'penis', 'piss', 'prick', 'pube', 'pussy', 'queer', 'scrotum',
+    'sex', 'shit', 'slut', 'smegma', 'terrorist', 'twat', 'vagina', 'wank', 'whore'
+))
+RESTRICTED_WORDS = frozenset(('ocf', 'ucb', 'berkeley', 'university'))
 
-    # TODO: check if kerberos principal already exists; skip this if so
-    with report_status('Creating', 'Created', 'Kerberos keytab'):
-        create_kerberos_principal_with_keytab(
-            request.user_name,
-            creds.kerberos_keytab,
-            creds.kerberos_principal,
-            password=decrypt_password(
-                request.encrypted_password,
-                RSA.importKey(open(creds.encryption_key).read()),
-            ),
-        )
+CREATE_PUBLIC_KEY = '''\
+-----BEGIN PUBLIC KEY-----
+MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA3xG2dczz2y+qc0AgTZ1L
+Jrun4RbcMf7z7AFqPqIQrtbuJprg6EQPHd2EDjMt9rJm929tTatjLu7TcNisq9lW
+AMU85c1nM6l4ce58mIXGzaB5yCGP0Qfcos5o00xJOmRVmxZGB5D2Jli+JbmEHPpo
+KbvI3DuNLNbS+MxXawudEDVj0xA86Iv8biHqq//xMD+SicOzN4ZrjKarT9MdQYL+
+JDNjiYba1ZiNLiqXeLGS2IVYAd88etX+V5gxAvl0bGHzgeHodutxUf46QCg7cmvm
+5lQsbiYUABiEsE1OejSEfb+7mtuhxu+MeVXCYr341axa0IHorj4qURxKOi/CTn5f
+zwIDAQAB
+-----END PUBLIC KEY-----'''
 
-    # TODO: check if LDAP entry already exists; skip this if so
-    with report_status('Finding', 'Found', 'first available UID'):
-        new_uid = _get_first_available_uid()
-
-    dn = 'uid={user},{base_people}'.format(
-        user=request.user_name,
-        base_people=constants.OCF_LDAP_PEOPLE,
-    )
-    attrs = {
-        'objectClass': ['ocfAccount', 'account', 'posixAccount'],
-        'cn': [request.real_name],
-        'uidNumber': [str(new_uid)],
-        'gidNumber': [str(getgrnam('ocf').gr_gid)],
-        'homeDirectory': [utils.home_dir(request.user_name)],
-        'loginShell': ['/bin/bash'],
-        'mail': [request.email],
-        'userPassword': ['{SASL}' + request.user_name + '@OCF.BERKELEY.EDU'],
-        'creationTime': [datetime.now().strftime('%Y%m%d%H%M%SZ')],
-    }
-    if request.calnet_uid:
-        attrs['calnetUid'] = [str(request.calnet_uid)]
-    else:
-        attrs['callinkOid'] = [str(request.callink_oid)]
-
-    with report_status('Creating', 'Created', 'LDAP entry'):
-        create_ldap_entry_with_keytab(
-            dn, attrs, creds.kerberos_keytab, creds.kerberos_principal,
-        )
-
-        # invalidate passwd cache so that we can immediately chown files
-        # XXX: sometimes this fails, but that's okay because it means
-        # nscd isn't running anyway
-        call(('sudo', 'nscd', '-i', 'passwd'))
-
-    with report_status('Creating', 'Created', 'home and web directories'):
-        create_home_dir(request.user_name)
-        create_web_dir(request.user_name)
-
-    send_created_mail(request)
-    # TODO: logging to syslog, files
+# Inclusive endpoints
+RESERVED_UID_RANGES = [
+    # 61184-65519 are the systemd dymanic users
+    # 65534 is the nobody user
+    # 65535 is the invalid user
+    # we reserve the gaps between these for extra safety
+    (61184, 65535),
+]
 
 
-def _get_first_available_uid():
+def _get_first_available_uid(known_uid=_KNOWN_UID):
     """Return the first available UID number.
 
     Searches our entire People ou in order to find it. It seems like there
     should be a better way to do this, but quick searches don't show any.
 
     We hard-code a value we know has already been reached and only select
-    entries greater than that for performance. We then use a module-level
-    dict to cache our output for the next function call.
+    entries greater than that for performance. This value can then be cached
+    and passed back in to make subsequent calls faster.
     """
-    min_uid = _cache['known_uid']
     with ldap_ocf() as c:
         c.search(
-            constants.OCF_LDAP_PEOPLE,
-            '(uidNumber>={KNOWN_MIN})'.format(KNOWN_MIN=min_uid),
+            OCF_LDAP_PEOPLE,
+            '(uidNumber>={KNOWN_MIN})'.format(KNOWN_MIN=known_uid),
             attributes=['uidNumber'],
         )
-        uids = [int(entry['attributes']['uidNumber'][0]) for entry in c.response]
+        uids = [int(entry['attributes']['uidNumber']) for entry in c.response]
     if uids:
         max_uid = max(uids)
-        _cache['known_uid'] = max_uid
     else:
         # If cached UID is later deleted, LDAP response will be empty.
-        max_uid = min_uid
-    return max_uid + 1
+        max_uid = known_uid
+
+    assert all(start <= end for start, end in RESERVED_UID_RANGES)
+    next_uid = max_uid + 1
+    for start, end in sorted(RESERVED_UID_RANGES):
+        if start <= next_uid <= end:
+            next_uid = end + 1
+
+    return next_uid
+
+
+def create_account(request, creds, report_status, known_uid=_KNOWN_UID):
+    """Create an account as idempotently as possible.
+
+    :param known_uid: where to start searching for unused UIDs (see
+        _get_first_available_uid)
+    :return: the UID of the newly created account
+    """
+    # TODO: better docstring
+
+    if get_kerberos_principal_with_keytab(
+        request.user_name,
+        creds.kerberos_keytab,
+        creds.kerberos_principal,
+    ):
+        report_status('kerberos principal already exists; skipping creation')
+    else:
+        with report_status('Creating', 'Created', 'Kerberos keytab'):
+            create_kerberos_principal_with_keytab(
+                request.user_name,
+                creds.kerberos_keytab,
+                creds.kerberos_principal,
+                password=decrypt_password(
+                    request.encrypted_password,
+                    RSA.importKey(open(creds.encryption_key).read()),
+                ),
+            )
+
+    if search.user_attrs(request.user_name):
+        report_status('LDAP entry already exists; skipping creation')
+    else:
+        with report_status('Finding', 'Found', 'first available UID'):
+            new_uid = _get_first_available_uid(known_uid)
+
+        dn = utils.dn_for_username(request.user_name)
+        attrs = {
+            'objectClass': ['ocfAccount', 'account', 'posixAccount'],
+            'cn': [request.real_name],
+            'uidNumber': new_uid,
+            'gidNumber': getgrnam('ocf').gr_gid,
+            'homeDirectory': utils.home_dir(request.user_name),
+            'loginShell': '/bin/bash',
+            'ocfEmail': request.user_name + '@ocf.berkeley.edu',
+            'mail': [request.email],
+            'userPassword': '{SASL}' + request.user_name + '@OCF.BERKELEY.EDU',
+            'creationTime': datetime.now(timezone.utc).astimezone(),
+        }
+        if request.calnet_uid:
+            attrs['calnetUid'] = request.calnet_uid
+        else:
+            attrs['callinkOid'] = request.callink_oid
+
+        with report_status('Creating', 'Created', 'LDAP entry'):
+            create_ldap_entry(
+                dn,
+                attrs,
+                keytab=creds.kerberos_keytab,
+                admin_principal=creds.kerberos_principal,
+            )
+
+            # invalidate passwd cache so that we can immediately chown files
+            # XXX: sometimes this fails, but that's okay because it means
+            # nscd isn't running anyway
+            call(('sudo', 'nscd', '-i', 'passwd'))
+
+    with report_status('Creating', 'Created', 'home and web directories'):
+        create_home_dir(request.user_name)
+        ensure_web_dir(request.user_name)
+
+    send_created_mail(request)
+    # TODO: logging to syslog, files
+
+    return new_uid
 
 
 def create_home_dir(user):
-    """Create home directory for user. Makes a directory with appropriate
-    permissions, then copies in OCF's skeleton dotfiles.
-    """
+    """Create home directory for user with appropriate permissions."""
     home = utils.home_dir(user)
     subprocess.check_call(
         ['sudo', 'install', '-d', '--mode=0700', '--group=ocf',
             '--owner=' + user, home])
 
 
-def create_web_dir(user):
-    """Create web directory for user with appropriate permissions.
+def ensure_web_dir(user):
+    """Ensure user has web directory with appropriate permissions and
+    public_html symlink.
 
     All users are given a working web directory and public_html symlink at
     account creation. They can later use `makehttp` to fix these if they bork
-    the permissions or symlink.
+    the permissions or symlink. If there is already a file in the user's home
+    folder named 'public_html', it will be renamed by appending a timestamp.
     """
     path = utils.web_dir(user)
 
-    # create web directory
+    # ensure web directory exists and has the right permissiosn
     subprocess.check_call([
         'sudo', 'install',
         '-d', '--mode=0755', '--group=ocf', '--owner=' + user,
@@ -135,81 +192,47 @@ def create_web_dir(user):
         path,
     ])
 
-    # symlink it from ~user/public_html
+    public_html_path = utils.public_html_path(user)
+
+    # rename any existing files named ~user/public_html
+    if os.path.exists(public_html_path) and os.path.realpath(public_html_path) != path:
+        timestamp = datetime.now().strftime('%m%d%Y-%H%M%S')
+        subprocess.check_call([
+            'sudo', 'mv', public_html_path, public_html_path + '.' + timestamp,
+        ])
+
+    # symlink web dir from ~user/public_html
     subprocess.check_call([
         'sudo', '-u', user,
-        'ln', '-fs', '--', path, os.path.join(utils.home_dir(user), 'public_html'),
+        'ln', '-fs', '--', path, public_html_path,
     ])
 
 
 def send_created_mail(request):
-    body = """Greetings from the Open Computing Facility,
-
-Your OCF account has been created and is ready for use! Welcome aboard!
-
-Your account name is: {request.user_name}
-
-As a brand-new OCF member, you're welcome to use any and all of our services.
-Some key highlights:
-
-    - Access to our swanky computer lab (171 MLK Student Union)
-    - 250 pages of free printing per semester
-    - Web hosting: https://www.ocf.berkeley.edu/~{request.user_name}/
-
-You can find out the cool things you can do on our wiki:
-https://ocf.io/wiki
-
-Keep in mind not to share your shiny new password with anyone, including OCF
-staffers. You can always reset it online:
-
-https://ocf.io/password
-
-Finally, we'd like to remind you that the OCF is run completely by student
-volunteers (people like you)! If you value the services we provide, check us
-out and consider getting involved!
-
-https://www.ocf.berkeley.edu/about/staff
-
-If you have any other questions, feel free to reply to this message!
-
-{signature}""".format(request=request,
-                      signature=constants.MAIL_SIGNATURE)
-
+    body = jinja_mail_env.get_template(
+        'account/mail_templates/account-created.jinja',
+    ).render(
+        request=request,
+        semesterly_quota=SEMESTERLY_QUOTA,
+    )
     send_mail(request.email, '[OCF] Your account has been created!', body)
 
 
 def send_rejected_mail(request, reason):
-    body = """Greetings from the Open Computing Facility,
-
-Your OCF account, {request.user_name} has been rejected for the following reason:
-
-{reason}
-
-For information about account eligibility, see:
-
-https://wiki.ocf.berkeley.edu/membership/
-
-If you have any other questions or problems, feel free to contact us by
-replying to this message.
-
-{signature}""".format(request=request,
-                      reason=reason,
-                      signature=constants.MAIL_SIGNATURE)
-
-    send_mail(
-        request.email, '[OCF] Your account request has been rejected', body)
+    body = jinja_mail_env.get_template(
+        'account/mail_templates/account-rejected.jinja',
+    ).render(request=request, reason=reason)
+    send_mail(request.email, '[OCF] Your account request has been rejected', body)
 
 
-class ValidationWarning(ValueError):
+class ValidationWarning(Exception):
     """Warning exception raised by validators when a staff member needs to
     manually approve an account."""
-    pass
 
 
-class ValidationError(ValueError):
+class ValidationError(Exception):
     """Error exception raised by validators when a request should be
     rejected."""
-    pass
 
 
 def validate_callink_oid(oid):
@@ -257,11 +280,12 @@ def validate_calnet_uid(uid):
     if not attrs:
         raise ValidationError("CalNet UID can't be found in university LDAP.")
 
+    # TODO: Uncomment when we get privileged LDAP bind.
     # check if user is eligible for an account
-    affiliations = attrs['berkeleyEduAffiliations']
-    if not eligible_for_account(affiliations):
-        raise ValidationWarning(
-            'Affiliate type not eligible for account: ' + str(affiliations))
+    # affiliations = attrs['berkeleyEduAffiliations']
+    # if not eligible_for_account(affiliations):
+    #    raise ValidationWarning(
+    #        'Affiliate type not eligible for account: ' + str(affiliations))
 
 
 def eligible_for_account(affiliations):
@@ -330,13 +354,13 @@ def validate_username(username, realname):
 
     if similarity_heuristic(realname, username) > SIMILARITY_THRESHOLD:
         raise ValidationWarning(
-            'Username {} not based on real name {}'.format(username, realname))
+            'Username {} not based on real name {}.'.format(username, realname))
 
-    if any(word in username for word in constants.BAD_WORDS):
-        raise ValidationWarning('Username {} contains bad words'.format(username))
+    if any(word in username for word in BAD_WORDS):
+        raise ValidationWarning('Username {} contains bad words.'.format(username))
 
-    if any(word in username for word in constants.RESTRICTED_WORDS):
-        raise ValidationWarning('Username {} contains restricted words'.format(username))
+    if any(word in username for word in RESTRICTED_WORDS):
+        raise ValidationWarning('Username {} contains restricted words.'.format(username))
 
 
 def similarity_heuristic(realname, username):
@@ -360,7 +384,7 @@ def similarity_heuristic(realname, username):
     max_words = 8
     max_iterations = math.factorial(max_words)
 
-    words = re.findall('\w+', realname)
+    words = re.findall(r'\w+', realname)
     initials = [word[0] for word in words]
 
     if len(words) > max_words:
@@ -503,10 +527,3 @@ class NewAccountRequest(namedtuple('NewAccountRequest', [
             field: getattr(self, field)
             for field in self._fields if field != 'encrypted_password'
         }
-
-
-# We use a module-level dict to "cache" across function calls.
-# TODO: This prevents importing this module when LDAP is unavailable.
-# https://github.com/ocf/ocfweb/issues/103
-_cache = {'known_uid': 37500}
-_get_first_available_uid()
