@@ -1,12 +1,17 @@
 """Module containing account management methods, such as password changing, but
 not account creation (since it's too large)."""
+import shlex
+
 import pexpect
 
 import ocflib.account.search as search
+import ocflib.account.utils as utils
 import ocflib.account.validators as validators
-import ocflib.constants as constants
+import ocflib.infra.ldap as ldap_ocf
+import ocflib.misc as misc
 import ocflib.misc.mail as mail
-import ocflib.misc.shell as shell
+
+KADMIN_PATH = '/usr/bin/kadmin'
 
 
 def change_password_with_staffer(username, password, principal,
@@ -21,9 +26,9 @@ def change_password_with_staffer(username, password, principal,
 
     # try changing using kadmin pexpect
     cmd = '{kadmin_path} -p {principal} cpw {username}'.format(
-        kadmin_path=shell.escape_arg(constants.KADMIN_PATH),
-        principal=shell.escape_arg(principal),
-        username=shell.escape_arg(username))
+        kadmin_path=shlex.quote(KADMIN_PATH),
+        principal=shlex.quote(principal),
+        username=shlex.quote(username))
 
     child = pexpect.spawn(cmd, timeout=10)
 
@@ -59,10 +64,10 @@ def change_password_with_keytab(username, password, keytab, principal, comment=N
 
     # try changing using kadmin pexpect
     cmd = '{kadmin_path} -K {keytab} -p {principal} cpw {username}'.format(
-        kadmin_path=shell.escape_arg(constants.KADMIN_PATH),
-        keytab=shell.escape_arg(keytab),
-        principal=shell.escape_arg(principal),
-        username=shell.escape_arg(username))
+        kadmin_path=shlex.quote(KADMIN_PATH),
+        keytab=shlex.quote(keytab),
+        principal=shlex.quote(principal),
+        username=shlex.quote(username))
 
     child = pexpect.spawn(cmd, timeout=10)
 
@@ -81,6 +86,30 @@ def change_password_with_keytab(username, password, keytab, principal, comment=N
     _notify_password_change(username, comment=comment)
 
 
+def modify_ldap_attributes(username, attributes, **kwargs):
+    """Adds or modifies arbitrary attributes of a user's LDAP record subject to
+    minor validation beyond the LDAP schema.
+
+    At the moment, the only attribute that benefits from extra validation is
+    the 'loginShell' attribute.
+    """
+
+    login_shell = attributes.get('loginShell', None)
+    if login_shell is not None:
+        if not isinstance(login_shell, str):
+            raise ValueError('Login shell must be a string')
+
+        if not misc.validators.valid_login_shell(login_shell):
+            raise ValueError("Invalid login shell '{}'".format(login_shell))
+
+    ldap_ocf.modify_ldap_entry(
+        utils.dn_for_username(username),
+        attributes,
+        **kwargs
+    )
+    # TODO: Add trailing comma to **kwargs for Python 3.6+
+
+
 def _notify_password_change(username, comment=None):
     """Send email about a password change.
 
@@ -97,14 +126,14 @@ def _notify_password_change(username, comment=None):
 Just a quick heads up that your Open Computing Facility account password was
 just reset, hopefully by you.
 {comment_line}
-As a a reminder, your OCF username is: {username}
+As a reminder, your OCF username is: {username}
 
 If you're not sure why this happened, please reply to this email ASAP.
 
 {signature}""".format(
         name=name,
         username=username,
-        signature=constants.MAIL_SIGNATURE,
+        signature=mail.MAIL_SIGNATURE,
         comment_line=('\n' + comment + '\n') if comment else '',
     )
 
