@@ -22,10 +22,13 @@ appropriate broker and backend URL (probably Redis).
     # You can immediately resolve it with result.wait(timeout=5), or grab
     # result.id and fetch it later.
 """
+import datetime
+import socket
 from collections import namedtuple
 from contextlib import contextmanager
 
 import redis
+import sqlalchemy.exc
 from redis.exceptions import LockError
 from sqlalchemy import Boolean
 from sqlalchemy import Column
@@ -194,14 +197,21 @@ def get_tasks(celery_app, credentials=None):
             if request.handle_warnings == NewAccountRequest.WARNINGS_SUBMIT:
                 stored_request = StoredNewAccountRequest.from_request(request, str(warnings))
 
-                with get_session() as session:
-                    session.add(stored_request)  # TODO: error handling
-                    session.commit()
+                try:
+                    with get_session() as session:
+                        session.add(stored_request)  # TODO: error handling
+                        session.commit()
+                except sqlalchemy.exc.IntegrityError:
+                    # If there's an integrity constraint, it's okay -- the
+                    # account was already submitted, so we can still return a
+                    # "pending" response.
+                    pass
+                else:
+                    dispatch_event(
+                        'ocflib.account_submitted',
+                        request=dict(request.to_dict(), reasons=warnings),
+                    )
 
-                dispatch_event(
-                    'ocflib.account_submitted',
-                    request=dict(request.to_dict(), reasons=warnings),
-                )
                 return NewAccountResponse(
                     status=NewAccountResponse.PENDING,
                     errors=warnings,
@@ -227,16 +237,23 @@ def get_tasks(celery_app, credentials=None):
             # status reporting
             status = []
 
-            def _report_status(line):
-                """Update task status by adding the given line."""
-                status.append(line)
-                create_account.update_state(meta={'status': status})
+            class report_status:
 
-            @contextmanager
-            def report_status(start, stop, task):
-                _report_status(start + ' ' + task)
-                yield
-                _report_status(stop + ' ' + task)
+                def __init__(self, *args):
+                    if len(args) == 1:
+                        self(*args)
+                    else:
+                        self.start, self.stop, self.task = args
+
+                def __call__(self, line):
+                    status.append(line)
+                    create_account.update_state(meta={'status': status})
+
+                def __enter__(self, *args):
+                    self(self.start + ' ' + self.task)
+
+                def __exit__(self, *args):
+                    self(self.stop + ' ' + self.task)
 
             with report_status('Validating', 'Validated', 'request'), \
                     get_session() as session:
@@ -250,7 +267,13 @@ def get_tasks(celery_app, credentials=None):
                 )
 
             # actual account creation
-            real_create_account(request, credentials, report_status)
+            kwargs = {}
+            known_uid = r.get('known_uid')
+            if known_uid:
+                kwargs['known_uid'] = int(known_uid)
+            new_uid = real_create_account(request, credentials, report_status, **kwargs)
+            r.set('known_uid', new_uid)
+
             dispatch_event('ocflib.account_created', request=request.to_dict())
             return NewAccountResponse(
                 status=NewAccountResponse.CREATED,
@@ -309,6 +332,14 @@ def get_tasks(celery_app, credentials=None):
             comment=comment,
         )
 
+    @celery_app.task
+    def status():
+        """A testing route."""
+        return {
+            'now': datetime.datetime.now().isoformat(),
+            'host': socket.getfqdn(),
+        }
+
     return _AccountSubmissionTasks(
         validate_then_create_account=validate_then_create_account,
         create_account=create_account,
@@ -316,8 +347,10 @@ def get_tasks(celery_app, credentials=None):
         approve_request=approve_request,
         reject_request=reject_request,
         change_password=change_password,
+        status=status,
     )
 
+
 _AccountSubmissionTasks = namedtuple('AccountSubmissionTasks', [
     'validate_then_create_account',
     'create_account',
@@ -325,6 +358,7 @@ _AccountSubmissionTasks = namedtuple('AccountSubmissionTasks', [
     'approve_request',
     'reject_request',
     'change_password',
+    'status',
 ])
 
 AccountCreationCredentials = namedtuple('AccountCreationCredentials', [
