@@ -5,10 +5,11 @@ from xml.etree import ElementTree
 import requests
 
 import ocflib.account.search as search
+from ocflib.ucb.directory import name_by_calnet_uid
 
 
 _API = {
-    'BASE': 'https://studentservices.berkeley.edu/WebServices/StudentGroupServiceV2/Service.asmx',  # noqa
+    'BASE': 'https://studentgroupservice.sait-west.berkeley.edu/service.asmx',
     'SERVICE': {
         'ORGS': 'CalLinkOrganizations',
         'SIGNATORIES_BY_OID': 'CalLinkGroupSignatories',
@@ -18,7 +19,7 @@ _API = {
 }
 
 
-def list_groups(name=None, oid=None, status=None, type=None, category=None):
+def list_groups(name='', oid='', status='', type='', category=''):
     """Return groups by a general CalLink search.
 
     >>> list_groups(name="facility")
@@ -56,11 +57,11 @@ def list_groups(name=None, oid=None, status=None, type=None, category=None):
         return {oid: name for oid, name in map(parse, xml_groups)}
 
     return _get_osl({
-        'name': name or '',
-        'organizationId': str(oid) if oid else '',
-        'status': status or '',
-        'type': type or '',
-        'category': category or '',
+        'name': name,
+        'organizationId': str(oid),
+        'status': status,
+        'type': type,
+        'category': category,
     }, _API['SERVICE']['ORGS'], parser)
 
 
@@ -103,13 +104,7 @@ def signatories_for_group(oid):
     def parser(root):
         def parse(student):
             uid = int(student.findtext('Username'))
-
-            attrs = search.user_attrs_ucb(uid)
-            name = None
-
-            if attrs:
-                name = attrs.get('displayName', [None])[0]
-
+            name = name_by_calnet_uid(uid)
             users = search.users_by_calnet_uid(uid)
             return uid, {'name': name, 'accounts': users}
 
@@ -152,17 +147,14 @@ def _get_osl(query, service, parser):
 
     url = '{}/{}?{}'.format(_API['BASE'], service, urlencode(query))
 
-    r = requests.get(url)
+    r = requests.get(url, timeout=20)
     return _parse_osl(ElementTree.fromstring(r.text), parser)
 
 
 def _parse_osl(root, parser):
     """Assemble Python dictionaries of groups from XML document"""
     if root.findtext('Succeeded') == 'false':
-        try:
-            error_reason = root.findtext('Reason')
-        except:
-            error_reason = 'unknown reason'
+        error_reason = root.findtext('Reason', default='unknown reason')
         raise Exception('Lookup failed: ' + error_reason)
 
     return parser(root)
